@@ -12,21 +12,6 @@ typedef struct { PyObject_HEAD } PyMethodObject;
 typedef struct { PyObject_HEAD } PyListObject;
 typedef struct { PyObject_HEAD } PyLongObject;
 typedef struct { PyObject_HEAD } PyBaseExceptionObject;
-
-/* hack for https://bugs.python.org/issue29943 */
-
-PyAPI_FUNC(int) PyPySlice_GetIndicesEx(PyObject *arg0,
-                    Signed arg1, Signed *arg2,
-                    Signed *arg3, Signed *arg4, Signed *arg5);
-#ifdef __GNUC__
-__attribute__((__unused__))
-#endif
-static int PySlice_GetIndicesEx(PyObject *arg0, Py_ssize_t arg1,
-        Py_ssize_t *arg2, Py_ssize_t *arg3, Py_ssize_t *arg4,
-        Py_ssize_t *arg5) {
-    return PyPySlice_GetIndicesEx(arg0, arg1, arg2, arg3,
-                arg4, arg5);
-}
 #define PyAnySet_Check PyPyAnySet_Check
 PyAPI_FUNC(int) PyAnySet_Check(struct _object *arg0);
 #define PyAnySet_CheckExact PyPyAnySet_CheckExact
@@ -83,6 +68,16 @@ PyAPI_FUNC(struct _object *) PyCMethod_New(struct PyMethodDef *arg0, struct _obj
 PyAPI_FUNC(struct _object *) PyCallIter_New(struct _object *arg0, struct _object *arg1);
 #define PyCallable_Check PyPyCallable_Check
 PyAPI_FUNC(int) PyCallable_Check(struct _object *arg0);
+#define PyCapsule_New PyPyCapsule_New
+PyAPI_FUNC(PyObject *) PyCapsule_New(void * arg0, char const * arg1, PyCapsule_Destructor arg2);
+#define PyCapsule_SetContext PyPyCapsule_SetContext
+PyAPI_FUNC(int) PyCapsule_SetContext(PyObject * arg0, void * arg1);
+#define PyCapsule_SetDestructor PyPyCapsule_SetDestructor
+PyAPI_FUNC(int) PyCapsule_SetDestructor(PyObject * arg0, PyCapsule_Destructor arg1);
+#define PyCapsule_SetName PyPyCapsule_SetName
+PyAPI_FUNC(int) PyCapsule_SetName(PyObject * arg0, char const * arg1);
+#define PyCapsule_SetPointer PyPyCapsule_SetPointer
+PyAPI_FUNC(int) PyCapsule_SetPointer(PyObject * arg0, void * arg1);
 #define PyClassMethod_New PyPyClassMethod_New
 PyAPI_FUNC(struct _object *) PyClassMethod_New(struct _object *arg0);
 #define PyCode_Addr2Line PyPyCode_Addr2Line
@@ -141,6 +136,8 @@ PyAPI_FUNC(int) PyDateTime_DATE_GET_MICROSECOND(void *arg0);
 PyAPI_FUNC(int) PyDateTime_DATE_GET_MINUTE(void *arg0);
 #define PyDateTime_DATE_GET_SECOND PyPyDateTime_DATE_GET_SECOND
 PyAPI_FUNC(int) PyDateTime_DATE_GET_SECOND(void *arg0);
+#define PyDateTime_DATE_GET_TZINFO PyPyDateTime_DATE_GET_TZINFO
+PyAPI_FUNC(struct _object *) PyDateTime_DATE_GET_TZINFO(void *arg0);
 #define PyDateTime_DELTA_GET_DAYS PyPyDateTime_DELTA_GET_DAYS
 PyAPI_FUNC(int) PyDateTime_DELTA_GET_DAYS(void *arg0);
 #define PyDateTime_DELTA_GET_MICROSECONDS PyPyDateTime_DELTA_GET_MICROSECONDS
@@ -167,6 +164,8 @@ PyAPI_FUNC(int) PyDateTime_TIME_GET_MICROSECOND(void *arg0);
 PyAPI_FUNC(int) PyDateTime_TIME_GET_MINUTE(void *arg0);
 #define PyDateTime_TIME_GET_SECOND PyPyDateTime_TIME_GET_SECOND
 PyAPI_FUNC(int) PyDateTime_TIME_GET_SECOND(void *arg0);
+#define PyDateTime_TIME_GET_TZINFO PyPyDateTime_TIME_GET_TZINFO
+PyAPI_FUNC(struct _object *) PyDateTime_TIME_GET_TZINFO(void *arg0);
 #define PyDate_Check PyPyDate_Check
 PyAPI_FUNC(int) PyDate_Check(struct _object *arg0);
 #define PyDate_CheckExact PyPyDate_CheckExact
@@ -265,8 +264,8 @@ PyAPI_FUNC(struct _object *) PyErr_SetFromErrno(struct _object *arg0);
 PyAPI_FUNC(struct _object *) PyErr_SetFromErrnoWithFilename(struct _object *arg0, const char *arg1);
 #define PyErr_SetFromErrnoWithFilenameObject PyPyErr_SetFromErrnoWithFilenameObject
 PyAPI_FUNC(struct _object *) PyErr_SetFromErrnoWithFilenameObject(struct _object *arg0, struct _object *arg1);
-#define PyErr_SetInterrupt PyPyErr_SetInterrupt
-PyAPI_FUNC(void) PyErr_SetInterrupt(void);
+#define PyErr_SetFromErrnoWithFilenameObjects PyPyErr_SetFromErrnoWithFilenameObjects
+PyAPI_FUNC(struct _object *) PyErr_SetFromErrnoWithFilenameObjects(struct _object *arg0, struct _object *arg1, struct _object *arg2);
 #define PyErr_SetNone PyPyErr_SetNone
 PyAPI_FUNC(void) PyErr_SetNone(struct _object *arg0);
 #define PyErr_SetObject PyPyErr_SetObject
@@ -335,6 +334,10 @@ PyAPI_FUNC(int) PyFile_WriteString(const char *arg0, struct _object *arg1);
 PyAPI_FUNC(double) PyFloat_AS_DOUBLE(void *arg0);
 #define PyFloat_AsDouble PyPyFloat_AsDouble
 PyAPI_FUNC(double) PyFloat_AsDouble(struct _object *arg0);
+#define PyFloat_Check PyPyFloat_Check
+PyAPI_FUNC(int) PyFloat_Check(void * arg0);
+#define PyFloat_CheckExact PyPyFloat_CheckExact
+PyAPI_FUNC(int) PyFloat_CheckExact(void * arg0);
 #define PyFloat_FromDouble PyPyFloat_FromDouble
 PyAPI_FUNC(struct _object *) PyFloat_FromDouble(double arg0);
 #define PyFloat_FromString PyPyFloat_FromString
@@ -353,6 +356,12 @@ PyAPI_FUNC(int) PyFunction_Check(void * arg0);
 PyAPI_FUNC(int) PyFunction_CheckExact(void * arg0);
 #define PyFunction_GetCode PyPyFunction_GetCode
 PyAPI_FUNC(struct _object *) PyFunction_GetCode(struct _object *arg0);
+#define PyGC_Disable PyPyGC_Disable
+PyAPI_FUNC(int) PyGC_Disable(void);
+#define PyGC_Enable PyPyGC_Enable
+PyAPI_FUNC(int) PyGC_Enable(void);
+#define PyGC_IsEnabled PyPyGC_IsEnabled
+PyAPI_FUNC(int) PyGC_IsEnabled(void);
 #define PyGILState_Check PyPyGILState_Check
 PyAPI_FUNC(int) PyGILState_Check(void);
 #define PyGILState_Ensure PyPyGILState_Ensure
@@ -383,8 +392,6 @@ PyAPI_FUNC(PyObject *) PyImport_ImportModuleLevelObject(PyObject * arg0, PyObjec
 PyAPI_FUNC(struct _object *) PyImport_ImportModuleNoBlock(const char *arg0);
 #define PyImport_ReloadModule PyPyImport_ReloadModule
 PyAPI_FUNC(struct _object *) PyImport_ReloadModule(struct _object *arg0);
-#define PyIndex_Check PyPyIndex_Check
-PyAPI_FUNC(int) PyIndex_Check(struct _object *arg0);
 #define PyInstanceMethod_Check PyPyInstanceMethod_Check
 PyAPI_FUNC(int) PyInstanceMethod_Check(struct _object *arg0);
 #define PyInstanceMethod_Function PyPyInstanceMethod_Function
@@ -393,6 +400,8 @@ PyAPI_FUNC(struct _object *) PyInstanceMethod_Function(struct _object *arg0);
 PyAPI_FUNC(struct _object *) PyInstanceMethod_GET_FUNCTION(struct _object *arg0);
 #define PyInstanceMethod_New PyPyInstanceMethod_New
 PyAPI_FUNC(struct _object *) PyInstanceMethod_New(struct _object *arg0);
+#define PyInterpreterState_GetID PyPyInterpreterState_GetID
+PyAPI_FUNC(Signed) PyInterpreterState_GetID(PyInterpreterState *arg0);
 #define PyInterpreterState_Head PyPyInterpreterState_Head
 PyAPI_FUNC(PyInterpreterState *) PyInterpreterState_Head(void);
 #define PyInterpreterState_Next PyPyInterpreterState_Next
@@ -401,6 +410,8 @@ PyAPI_FUNC(PyInterpreterState *) PyInterpreterState_Next(PyInterpreterState *arg
 PyAPI_FUNC(int) PyIter_Check(struct _object *arg0);
 #define PyIter_Next PyPyIter_Next
 PyAPI_FUNC(struct _object *) PyIter_Next(struct _object *arg0);
+#define PyIter_Send PyPyIter_Send
+PyAPI_FUNC(int) PyIter_Send(struct _object *arg0, struct _object *arg1, struct _object **arg2);
 #define PyList_Append PyPyList_Append
 PyAPI_FUNC(int) PyList_Append(struct _object *arg0, struct _object *arg1);
 #define PyList_AsTuple PyPyList_AsTuple
@@ -533,8 +544,12 @@ PyAPI_FUNC(struct _object *) PyModule_Create2(struct PyModuleDef *arg0, int arg1
 PyAPI_FUNC(int) PyModule_ExecDef(struct _object *arg0, struct PyModuleDef *arg1);
 #define PyModule_GetDict PyPyModule_GetDict
 PyAPI_FUNC(struct _object *) PyModule_GetDict(struct _object *arg0);
+#define PyModule_GetFilenameObject PyPyModule_GetFilenameObject
+PyAPI_FUNC(struct _object *) PyModule_GetFilenameObject(struct _object *arg0);
 #define PyModule_GetName PyPyModule_GetName
 PyAPI_FUNC(char *) PyModule_GetName(struct _object *arg0);
+#define PyModule_GetNameObject PyPyModule_GetNameObject
+PyAPI_FUNC(struct _object *) PyModule_GetNameObject(struct _object *arg0);
 #define PyModule_New PyPyModule_New
 PyAPI_FUNC(struct _object *) PyModule_New(const char *arg0);
 #define PyModule_NewObject PyPyModule_NewObject
@@ -547,8 +562,6 @@ PyAPI_FUNC(struct _object *) PyNumber_Add(struct _object *arg0, struct _object *
 PyAPI_FUNC(struct _object *) PyNumber_And(struct _object *arg0, struct _object *arg1);
 #define PyNumber_AsSsize_t PyPyNumber_AsSsize_t
 PyAPI_FUNC(Signed) PyNumber_AsSsize_t(struct _object *arg0, struct _object *arg1);
-#define PyNumber_Check PyPyNumber_Check
-PyAPI_FUNC(int) PyNumber_Check(struct _object *arg0);
 #define PyNumber_Divide PyPyNumber_Divide
 PyAPI_FUNC(struct _object *) PyNumber_Divide(struct _object *arg0, struct _object *arg1);
 #define PyNumber_Divmod PyPyNumber_Divmod
@@ -637,8 +650,16 @@ PyAPI_FUNC(int) PyObject_AsFileDescriptor(struct _object *arg0);
 PyAPI_FUNC(PyObject *) PyObject_Bytes(PyObject * arg0);
 #define PyObject_Call PyPyObject_Call
 PyAPI_FUNC(struct _object *) PyObject_Call(struct _object *arg0, struct _object *arg1, struct _object *arg2);
+#define PyObject_CallMethodNoArgs PyPyObject_CallMethodNoArgs
+PyAPI_FUNC(struct _object *) PyObject_CallMethodNoArgs(struct _object *arg0, struct _object *arg1);
+#define PyObject_CallMethodOneArg PyPyObject_CallMethodOneArg
+PyAPI_FUNC(struct _object *) PyObject_CallMethodOneArg(struct _object *arg0, struct _object *arg1, struct _object *arg2);
+#define PyObject_CallNoArgs PyPyObject_CallNoArgs
+PyAPI_FUNC(struct _object *) PyObject_CallNoArgs(struct _object *arg0);
 #define PyObject_CallObject PyPyObject_CallObject
 PyAPI_FUNC(struct _object *) PyObject_CallObject(struct _object *arg0, struct _object *arg1);
+#define PyObject_CallOneArg PyPyObject_CallOneArg
+PyAPI_FUNC(struct _object *) PyObject_CallOneArg(struct _object *arg0, struct _object *arg1);
 #define PyObject_Calloc PyPyObject_Calloc
 PyAPI_FUNC(void *) PyObject_Calloc(Unsigned arg0, Unsigned arg1);
 #define PyObject_ClearWeakRefs PyPyObject_ClearWeakRefs
@@ -653,6 +674,10 @@ PyAPI_FUNC(int) PyObject_DelItem(struct _object *arg0, struct _object *arg1);
 PyAPI_FUNC(struct _object *) PyObject_Dir(struct _object *arg0);
 #define PyObject_Format PyPyObject_Format
 PyAPI_FUNC(struct _object *) PyObject_Format(struct _object *arg0, struct _object *arg1);
+#define PyObject_GC_IsFinalized PyPyObject_GC_IsFinalized
+PyAPI_FUNC(int) PyObject_GC_IsFinalized(struct _object *arg0);
+#define PyObject_GC_IsTracked PyPyObject_GC_IsTracked
+PyAPI_FUNC(int) PyObject_GC_IsTracked(struct _object *arg0);
 #define PyObject_GenericGetAttr PyPyObject_GenericGetAttr
 PyAPI_FUNC(struct _object *) PyObject_GenericGetAttr(struct _object *arg0, struct _object *arg1);
 #define PyObject_GenericGetDict PyPyObject_GenericGetDict
@@ -719,6 +744,8 @@ PyAPI_FUNC(struct _object *) PyObject_Unicode(struct _object *arg0);
 PyAPI_FUNC(PyObject *) PyObject_Vectorcall(PyObject * arg0, PyObject * const * arg1, size_t arg2, PyObject * arg3);
 #define PyObject_VectorcallDict PyPyObject_VectorcallDict
 PyAPI_FUNC(PyObject *) PyObject_VectorcallDict(PyObject * arg0, PyObject * const * arg1, size_t arg2, PyObject * arg3);
+#define PyObject_VectorcallMethod PyPyObject_VectorcallMethod
+PyAPI_FUNC(PyObject *) PyObject_VectorcallMethod(PyObject * arg0, PyObject * const * arg1, size_t arg2, PyObject * arg3);
 #define PyPyUnicode_Check PyPyUnicode_Check
 PyAPI_FUNC(int) PyPyUnicode_Check(void * arg0);
 #define PyPyUnicode_CheckExact PyPyUnicode_CheckExact
@@ -831,6 +858,8 @@ PyAPI_FUNC(PyThreadState *) PyThreadState_Get(void);
 PyAPI_FUNC(struct _object *) PyThreadState_GetDict(void);
 #define PyThreadState_New PyPyThreadState_New
 PyAPI_FUNC(PyThreadState *) PyThreadState_New(PyInterpreterState *arg0);
+#define PyThreadState_SetAsyncExc PyPyThreadState_SetAsyncExc
+PyAPI_FUNC(int) PyThreadState_SetAsyncExc(Unsigned arg0, struct _object *arg1);
 #define PyThreadState_Swap PyPyThreadState_Swap
 PyAPI_FUNC(PyThreadState *) PyThreadState_Swap(PyThreadState *arg0);
 #define PyThread_exit_thread PyPyThread_exit_thread
@@ -857,16 +886,14 @@ PyAPI_FUNC(Signed) PyTuple_Size(struct _object *arg0);
 PyAPI_FUNC(PyObject *) PyType_FromModuleAndSpec(PyObject * arg0, PyType_Spec * arg1, PyObject * arg2);
 #define PyType_FromSpecWithBases PyPyType_FromSpecWithBases
 PyAPI_FUNC(PyObject *) PyType_FromSpecWithBases(PyType_Spec * arg0, PyObject * arg1);
-#define PyType_GenericNew PyPyType_GenericNew
-PyAPI_FUNC(struct _object *) PyType_GenericNew(struct _typeobject *arg0, struct _object *arg1, struct _object *arg2);
 #define PyType_GetSlot PyPyType_GetSlot
 PyAPI_FUNC(void *) PyType_GetSlot(struct _typeobject *arg0, int arg1);
-#define PyType_IsSubtype PyPyType_IsSubtype
-PyAPI_FUNC(int) PyType_IsSubtype(struct _typeobject *arg0, struct _typeobject *arg1);
 #define PyType_Modified PyPyType_Modified
 PyAPI_FUNC(void) PyType_Modified(struct _typeobject *arg0);
 #define PyType_Ready PyPyType_Ready
 PyAPI_FUNC(int) PyType_Ready(struct _typeobject *arg0);
+#define PyUnicode_Append PyPyUnicode_Append
+PyAPI_FUNC(void) PyUnicode_Append(struct _object **arg0, struct _object *arg1);
 #define PyUnicode_AsASCIIString PyPyUnicode_AsASCIIString
 PyAPI_FUNC(struct _object *) PyUnicode_AsASCIIString(struct _object *arg0);
 #define PyUnicode_AsEncodedObject PyPyUnicode_AsEncodedObject
@@ -1031,6 +1058,8 @@ PyAPI_FUNC(int) Py_GetRecursionLimit(void);
 PyAPI_FUNC(char *) Py_GetVersion(void);
 #define Py_IncRef PyPy_IncRef
 PyAPI_FUNC(void) Py_IncRef(struct _object *arg0);
+#define Py_Is PyPy_Is
+PyAPI_FUNC(int) Py_Is(struct _object *arg0, struct _object *arg1);
 #define Py_IsInitialized PyPy_IsInitialized
 PyAPI_FUNC(int) Py_IsInitialized(void);
 #define Py_LeaveRecursiveCall PyPy_LeaveRecursiveCall
@@ -1046,37 +1075,37 @@ PyAPI_FUNC(void) Py_SetRecursionLimit(int arg0);
 #define Py_UNICODE_COPY PyPy_UNICODE_COPY
 PyAPI_FUNC(void) Py_UNICODE_COPY(wchar_t *arg0, wchar_t *arg1, Signed arg2);
 #define Py_UNICODE_ISALNUM PyPy_UNICODE_ISALNUM
-PyAPI_FUNC(int) Py_UNICODE_ISALNUM(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_ISALNUM(unsigned int arg0);
 #define Py_UNICODE_ISALPHA PyPy_UNICODE_ISALPHA
-PyAPI_FUNC(int) Py_UNICODE_ISALPHA(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_ISALPHA(unsigned int arg0);
 #define Py_UNICODE_ISDECIMAL PyPy_UNICODE_ISDECIMAL
-PyAPI_FUNC(int) Py_UNICODE_ISDECIMAL(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_ISDECIMAL(unsigned int arg0);
 #define Py_UNICODE_ISDIGIT PyPy_UNICODE_ISDIGIT
-PyAPI_FUNC(int) Py_UNICODE_ISDIGIT(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_ISDIGIT(unsigned int arg0);
 #define Py_UNICODE_ISLINEBREAK PyPy_UNICODE_ISLINEBREAK
-PyAPI_FUNC(int) Py_UNICODE_ISLINEBREAK(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_ISLINEBREAK(unsigned int arg0);
 #define Py_UNICODE_ISLOWER PyPy_UNICODE_ISLOWER
-PyAPI_FUNC(int) Py_UNICODE_ISLOWER(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_ISLOWER(unsigned int arg0);
 #define Py_UNICODE_ISNUMERIC PyPy_UNICODE_ISNUMERIC
-PyAPI_FUNC(int) Py_UNICODE_ISNUMERIC(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_ISNUMERIC(unsigned int arg0);
 #define Py_UNICODE_ISSPACE PyPy_UNICODE_ISSPACE
-PyAPI_FUNC(int) Py_UNICODE_ISSPACE(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_ISSPACE(unsigned int arg0);
 #define Py_UNICODE_ISTITLE PyPy_UNICODE_ISTITLE
-PyAPI_FUNC(int) Py_UNICODE_ISTITLE(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_ISTITLE(unsigned int arg0);
 #define Py_UNICODE_ISUPPER PyPy_UNICODE_ISUPPER
-PyAPI_FUNC(int) Py_UNICODE_ISUPPER(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_ISUPPER(unsigned int arg0);
 #define Py_UNICODE_TODECIMAL PyPy_UNICODE_TODECIMAL
-PyAPI_FUNC(int) Py_UNICODE_TODECIMAL(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_TODECIMAL(unsigned int arg0);
 #define Py_UNICODE_TODIGIT PyPy_UNICODE_TODIGIT
-PyAPI_FUNC(int) Py_UNICODE_TODIGIT(wchar_t arg0);
+PyAPI_FUNC(int) Py_UNICODE_TODIGIT(unsigned int arg0);
 #define Py_UNICODE_TOLOWER PyPy_UNICODE_TOLOWER
-PyAPI_FUNC(wchar_t) Py_UNICODE_TOLOWER(wchar_t arg0);
+PyAPI_FUNC(unsigned int) Py_UNICODE_TOLOWER(unsigned int arg0);
 #define Py_UNICODE_TONUMERIC PyPy_UNICODE_TONUMERIC
-PyAPI_FUNC(double) Py_UNICODE_TONUMERIC(wchar_t arg0);
+PyAPI_FUNC(double) Py_UNICODE_TONUMERIC(unsigned int arg0);
 #define Py_UNICODE_TOTITLE PyPy_UNICODE_TOTITLE
-PyAPI_FUNC(wchar_t) Py_UNICODE_TOTITLE(wchar_t arg0);
+PyAPI_FUNC(unsigned int) Py_UNICODE_TOTITLE(unsigned int arg0);
 #define Py_UNICODE_TOUPPER PyPy_UNICODE_TOUPPER
-PyAPI_FUNC(wchar_t) Py_UNICODE_TOUPPER(wchar_t arg0);
+PyAPI_FUNC(unsigned int) Py_UNICODE_TOUPPER(unsigned int arg0);
 #define _PyBytes_Eq _PyPyBytes_Eq
 PyAPI_FUNC(int) _PyBytes_Eq(struct _object *arg0, struct _object *arg1);
 #define _PyBytes_Join _PyPyBytes_Join
@@ -1107,6 +1136,10 @@ PyAPI_FUNC(struct _object *) _PyDict_GetItemStringWithError(struct _object *arg0
 PyAPI_FUNC(int) _PyDict_HasOnlyStringKeys(struct _object *arg0);
 #define _PyErr_WriteUnraisableMsg _PyPyErr_WriteUnraisableMsg
 PyAPI_FUNC(void) _PyErr_WriteUnraisableMsg(const char *arg0, struct _object *arg1);
+#define _PyEval_GetAsyncGenFinalizer _PyPyEval_GetAsyncGenFinalizer
+PyAPI_FUNC(struct _object *) _PyEval_GetAsyncGenFinalizer(void);
+#define _PyEval_GetAsyncGenFirstiter _PyPyEval_GetAsyncGenFirstiter
+PyAPI_FUNC(struct _object *) _PyEval_GetAsyncGenFirstiter(void);
 #define _PyEval_SliceIndex _PyPyEval_SliceIndex
 PyAPI_FUNC(int) _PyEval_SliceIndex(struct _object *arg0, Signed *arg1);
 #define _PyFloat_Unpack4 _PyPyFloat_Unpack4
@@ -1121,6 +1154,8 @@ PyAPI_FUNC(int) _PyImport_ReleaseLock(void);
 PyAPI_FUNC(struct _object *) _PyList_Extend(struct _object *arg0, struct _object *arg1);
 #define _PyLong_AsByteArrayO _PyPyLong_AsByteArrayO
 PyAPI_FUNC(int) _PyLong_AsByteArrayO(struct _object *arg0, unsigned char *arg1, Unsigned arg2, int arg3, int arg4);
+#define _PyLong_AsInt _PyPyLong_AsInt
+PyAPI_FUNC(int) _PyLong_AsInt(struct _object *arg0);
 #define _PyLong_FromByteArray _PyPyLong_FromByteArray
 PyAPI_FUNC(struct _object *) _PyLong_FromByteArray(const unsigned char *arg0, Unsigned arg1, int arg2, int arg3);
 #define _PyLong_NumBits _PyPyLong_NumBits
@@ -1129,8 +1164,6 @@ PyAPI_FUNC(Unsigned) _PyLong_NumBits(struct _object *arg0);
 PyAPI_FUNC(int) _PyLong_Sign(struct _object *arg0);
 #define _PyNamespace_New _PyPyNamespace_New
 PyAPI_FUNC(PyObject *) _PyNamespace_New(PyObject * arg0);
-#define _PyObject_CallNoArg _PyPyObject_CallNoArg
-PyAPI_FUNC(struct _object *) _PyObject_CallNoArg(struct _object *arg0);
 #define _PyObject_FastCall _PyPyObject_FastCall
 PyAPI_FUNC(PyObject *) _PyObject_FastCall(PyObject * arg0, PyObject * const * arg1, size_t arg2);
 #define _PyObject_GetDictPtr _PyPyObject_GetDictPtr
@@ -1164,7 +1197,7 @@ PyAPI_FUNC(int) _PyUnicode_EqualToASCIIString(struct _object *arg0, const char *
 #define _PyUnicode_Ready _PyPyUnicode_Ready
 PyAPI_FUNC(int) _PyUnicode_Ready(PyObject * arg0);
 #define _Py_HashDouble _PyPy_HashDouble
-PyAPI_FUNC(Signed) _Py_HashDouble(double arg0);
+PyAPI_FUNC(Signed) _Py_HashDouble(struct _object *arg0, double arg1);
 #define _Py_HashPointer _PyPy_HashPointer
 PyAPI_FUNC(Signed) _Py_HashPointer(void *arg0);
 #define _Py_IsFinalizing _PyPy_IsFinalizing
@@ -1215,6 +1248,8 @@ PyAPI_DATA(PyObject*) PyExc_ConnectionResetError;
 PyAPI_DATA(PyObject*) PyExc_DeprecationWarning;
 #define PyExc_EOFError PyPyExc_EOFError
 PyAPI_DATA(PyObject*) PyExc_EOFError;
+#define PyExc_EncodingWarning PyPyExc_EncodingWarning
+PyAPI_DATA(PyObject*) PyExc_EncodingWarning;
 #define PyExc_Exception PyPyExc_Exception
 PyAPI_DATA(PyObject*) PyExc_Exception;
 #define PyExc_FileExistsError PyPyExc_FileExistsError
@@ -1313,8 +1348,6 @@ PyAPI_DATA(PyObject*) PyExc_ValueError;
 PyAPI_DATA(PyObject*) PyExc_Warning;
 #define PyExc_ZeroDivisionError PyPyExc_ZeroDivisionError
 PyAPI_DATA(PyObject*) PyExc_ZeroDivisionError;
-#define PyType_Type PyPyType_Type
-PyAPI_DATA(PyTypeObject) PyType_Type;
 #define PyBytes_Type PyPyBytes_Type
 PyAPI_DATA(PyTypeObject) PyBytes_Type;
 #define PyUnicode_Type PyPyUnicode_Type
@@ -1323,6 +1356,10 @@ PyAPI_DATA(PyTypeObject) PyUnicode_Type;
 PyAPI_DATA(PyTypeObject) PyDict_Type;
 #define PyDictProxy_Type PyPyDictProxy_Type
 PyAPI_DATA(PyTypeObject) PyDictProxy_Type;
+#define PyDictValues_Type PyPyDictValues_Type
+PyAPI_DATA(PyTypeObject) PyDictValues_Type;
+#define PyDictKeys_Type PyPyDictKeys_Type
+PyAPI_DATA(PyTypeObject) PyDictKeys_Type;
 #define PyTuple_Type PyPyTuple_Type
 PyAPI_DATA(PyTypeObject) PyTuple_Type;
 #define PyList_Type PyPyList_Type
@@ -1359,6 +1396,8 @@ PyAPI_DATA(PyTypeObject) PyProperty_Type;
 PyAPI_DATA(PyTypeObject) PySlice_Type;
 #define PyStaticMethod_Type PyPyStaticMethod_Type
 PyAPI_DATA(PyTypeObject) PyStaticMethod_Type;
+#define PyClassMethod_Type PyPyClassMethod_Type
+PyAPI_DATA(PyTypeObject) PyClassMethod_Type;
 #define PyCFunction_Type PyPyCFunction_Type
 PyAPI_DATA(PyTypeObject) PyCFunction_Type;
 #define PyClassMethodDescr_Type PyPyClassMethodDescr_Type
@@ -1375,6 +1414,18 @@ PyAPI_DATA(PyTypeObject) PyWrapperDescr_Type;
 PyAPI_DATA(PyTypeObject) PyInstanceMethod_Type;
 #define PyBufferable_Type PyPyBufferable_Type
 PyAPI_DATA(PyTypeObject) PyBufferable_Type;
+#define PyReversed_Type PyPyReversed_Type
+PyAPI_DATA(PyTypeObject) PyReversed_Type;
+#define PyRange_Type PyPyRange_Type
+PyAPI_DATA(PyTypeObject) PyRange_Type;
+#define PyFunction_Type PyPyFunction_Type
+PyAPI_DATA(PyTypeObject) PyFunction_Type;
+#define PyMethod_Type PyPyMethod_Type
+PyAPI_DATA(PyTypeObject) PyMethod_Type;
+#define PyTraceBack_Type PyPyTraceBack_Type
+PyAPI_DATA(PyTypeObject) PyTraceBack_Type;
+#define PyCapsule_Type PyPyCapsule_Type
+PyAPI_DATA(PyTypeObject) PyCapsule_Type;
 
 #undef Signed    /* xxx temporary fix */
 #undef Unsigned  /* xxx temporary fix */
