@@ -1,8 +1,11 @@
-import sys
 import os
+import pathlib
+import sys
 import time
 import _thread
+import warnings
 import weakref
+import __pypy__
 
 try:
     from _pypy_openssl import ffi
@@ -16,7 +19,8 @@ except ImportError as e:
     raise ModuleNotFoundError(str(e) + msg, name='_cffi_ssl')
 
 from _cffi_ssl._stdssl.certificate import (_test_decode_cert,
-    _decode_certificate, _certificate_to_der)
+    _decode_certificate, _certificate_to_der, _PySSL_CertificateFromX509Stack,
+    newCertificate)
 from _cffi_ssl._stdssl.utility import (_str_with_len, _bytes_with_len,
     _str_to_ffi_buffer, _str_from_buf, _cstr_decode_fs)
 from _cffi_ssl._stdssl.error import (
@@ -29,6 +33,8 @@ from _cffi_ssl._stdssl.error import (SSL_ERROR_NONE,
         SSL_ERROR_ZERO_RETURN, SSL_ERROR_WANT_CONNECT,
         SSL_ERROR_EOF, SSL_ERROR_NO_SOCKET, SSL_ERROR_INVALID_ERROR_CODE,
         pyerr_write_unraisable)
+from _cffi_ssl._stdssl.certificate import PY_SSL_ENCODING_PEM as ENCODING_PEM
+from _cffi_ssl._stdssl.certificate import PY_SSL_ENCODING_DER as ENCODING_DER
 from _cffi_ssl._stdssl import error
 from _pypy_util_cffi import StackNew
 from select import select
@@ -70,8 +76,8 @@ VERIFY_DEFAULT = 0
 VERIFY_CRL_CHECK_LEAF = lib.X509_V_FLAG_CRL_CHECK
 VERIFY_CRL_CHECK_CHAIN = lib.X509_V_FLAG_CRL_CHECK | lib.X509_V_FLAG_CRL_CHECK_ALL
 VERIFY_X509_STRICT = lib.X509_V_FLAG_X509_STRICT
-if lib.Cryptography_HAS_X509_V_FLAG_TRUSTED_FIRST:
-    VERIFY_X509_TRUSTED_FIRST = lib.X509_V_FLAG_TRUSTED_FIRST
+VERIFY_ALLOW_PROXY_CERTS = lib.X509_V_FLAG_ALLOW_PROXY_CERTS
+VERIFY_X509_TRUSTED_FIRST = lib.X509_V_FLAG_TRUSTED_FIRST
 if lib.Cryptography_HAS_X509_CHECK_FLAG_NEVER_CHECK_SUBJECT:
     HOSTFLAG_NEVER_CHECK_SUBJECT = lib.X509_CHECK_FLAG_NEVER_CHECK_SUBJECT
 
@@ -112,13 +118,26 @@ if lib.Cryptography_HAS_SSL3_METHOD:
         lib.SSL_CTX_free(ctx)
         SSLv3_method_ok = True
 
+# Python custom selection of sensible cipher suites
+# @SECLEVEL=2: security level 2 with 112 bits minimum security (e.g. 2048 bits RSA key)
+# ECDH+*: enable ephemeral elliptic curve Diffie-Hellman
+# DHE+*: fallback to ephemeral finite field Diffie-Hellman
+# encryption order: AES AEAD (GCM), ChaCha AEAD, AES CBC
+# !aNULL:!eNULL: really no NULL ciphers
+# !aDSS: no authentication with discrete logarithm DSA algorithm
+# !SHA1: no weak SHA1 MAC
+# !AESCCM: no CCM mode, it's uncommon and slow
+#
+# Based on Hynek's excellent blog post (update 2021-02-11)
+# https://hynek.me/articles/hardening-your-web-servers-ssl-ciphers/
+PY_SSL_DEFAULT_CIPHER_STRING = b"@SECLEVEL=2:ECDH+AESGCM:ECDH+CHACHA20:ECDH+AES:DHE+AES:!aNULL:!eNULL:!aDSS:!SHA1:!AESCCM"
+ 
 PROTOCOL_SSLv23 = 2
 PROTOCOL_TLS    = PROTOCOL_SSLv23
 PROTOCOL_TLSv1    = 3
-if lib.Cryptography_HAS_TLSv1_2:
-    PROTOCOL_TLSv1 = 3
-    PROTOCOL_TLSv1_1 = 4
-    PROTOCOL_TLSv1_2 = 5
+PROTOCOL_TLSv1 = 3
+PROTOCOL_TLSv1_1 = 4
+PROTOCOL_TLSv1_2 = 5
 PROTOCOL_TLS_CLIENT = 0x10
 PROTOCOL_TLS_SERVER = 0x11
 HAS_SSLv2 = bool(lib.Cryptography_HAS_SSL2)
@@ -325,6 +344,13 @@ class _SSLSocket(object):
         ctx = sslctx.ctx
         self.owner = None
 
+        if (socket_type == SSL_SERVER and sslctx.protocol == PROTOCOL_TLS_CLIENT):
+            raise ssl_error("Cannot create a server socket with a "
+                            "PROTOCOL_TLS_CLIENT context", 0)
+        if (socket_type == SSL_CLIENT and sslctx.protocol == PROTOCOL_TLS_SERVER):
+            raise ssl_error("Cannot create a client socket with a "
+                            "PROTOCOL_TLS_SERVER context", 0)
+
         if server_hostname:
             self.server_hostname = server_hostname.decode('ascii', 'strict')
             if '\x00' in self.server_hostname:
@@ -363,7 +389,7 @@ class _SSLSocket(object):
                     # bpo-37428: OpenSSL does not ignore SSL_VERIFY_POST_HANDSHAKE.
                     # Set SSL_VERIFY_POST_HANDSHAKE flag only for server sockets and
                     # only in combination with SSL_VERIFY_PEER flag.
-                    mode = lib.SSL_CTX_get_verify_mode(lib.SSL_get_SSL_CTX(self.ssl))
+                    mode = lib.SSL_get_verify_mode(self.ssl)
                     if (mode & lib.SSL_VERIFY_PEER):
                         verify_cb = lib.SSL_get_verify_callback(self.ssl)
                         mode |= lib.SSL_VERIFY_POST_HANDSHAKE
@@ -401,10 +427,8 @@ class _SSLSocket(object):
 
     def __init__(self, sslctx):
         self.ctx = sslctx
-        self.peer_cert = ffi.NULL
         self.ssl = ffi.NULL
         self.shutdown_seen_zero = 0
-        self.handshake_done = 0
         self._owner = None
         self.server_hostname = None
         self.socket = None
@@ -432,7 +456,7 @@ class _SSLSocket(object):
                     raise ssl_error(None)
             else:
                 if not lib.X509_VERIFY_PARAM_set1_ip(
-                        param, lib.ASN1_STRING_data(ip), lib.ASN1_STRING_length(ip)):
+                        param, lib.ASN1_STRING_get0_data(ip), lib.ASN1_STRING_length(ip)):
                     raise ssl_error(None)
 
     @property
@@ -510,7 +534,7 @@ class _SSLSocket(object):
                 sockstate = SOCKET_OPERATION_OK
 
             if sockstate == SOCKET_HAS_TIMED_OUT:
-                raise socket.timeout("The handshake operation timed out")
+                raise TimeoutError("The handshake operation timed out")
             elif sockstate == SOCKET_HAS_BEEN_CLOSED:
                 raise SSLError("Underlying socket has been closed.")
             elif sockstate == SOCKET_TOO_LARGE_FOR_SELECT:
@@ -522,11 +546,10 @@ class _SSLSocket(object):
         if ret < 1:
             raise pyssl_error(self, ret)
 
-        self.handshake_done = 1
         return None
 
     def getpeercert(self, binary_mode):
-        if not self.handshake_done:
+        if not lib.SSL_is_init_finished(self.ssl):
             raise ValueError("handshake not done yet")
         peer_cert = lib.SSL_get_peer_certificate(self.ssl);
         if peer_cert == ffi.NULL:
@@ -536,12 +559,39 @@ class _SSLSocket(object):
             # return cert in DER-encoded format
             return _certificate_to_der(peer_cert)
         else:
-            verification = lib.SSL_CTX_get_verify_mode(lib.SSL_get_SSL_CTX(self.ssl))
+            verification = lib.SSL_get_verify_mode(self.ssl)
             if (verification & lib.SSL_VERIFY_PEER) == 0:
                 return {}
             else:
                 return _decode_certificate(peer_cert)
 
+    def get_verified_chain(self):
+        chain = lib.SSL_get0_verified_chain(self.ssl)
+        if not chain:
+            return None
+        return _PySSL_CertificateFromX509Stack(chain, 1)
+
+    def get_unverified_chain(self):
+        chain = lib.SSL_get_peer_cert_chain(self.ssl)
+        if not chain:
+            return None
+        retval = _PySSL_CertificateFromX509Stack(chain, 1)
+        # OpenSSL does not include peer cert for server side connections
+        if self.socket_type == SSL_SERVER:
+            peer = lib.SSL_get_peer_certificate(self.ssl)
+            if not peer:
+                peerobj = None
+            else:
+                try:
+                    peerobj = newCertificate(peer, 0)
+                except ValueError:
+                    lib.X509_free(peer)
+                    raise
+            retval.insert(0, peerobj)
+        return retval
+                    
+        
+
     def write(self, bytestring):
         return self._write_with_length(_str_to_ffi_buffer(bytestring), len(bytestring))
 
@@ -549,9 +599,6 @@ class _SSLSocket(object):
         sock = self.get_socket_or_connection_gone()
         ssl = self.ssl
 
-        if lgt > _MAX_INT:
-            raise OverflowError("string longer than %d bytes" % _MAX_INT)
-
         timeout = _socket_timeout(sock)
         if sock:
             nonblocking = timeout >= 0
@@ -565,15 +612,16 @@ class _SSLSocket(object):
 
         sockstate = _ssl_select(sock, 1, timeout)
         if sockstate == SOCKET_HAS_TIMED_OUT:
-            raise socket.timeout("The write operation timed out")
+            raise TimeoutError("The write operation timed out")
         elif sockstate == SOCKET_HAS_BEEN_CLOSED:
             raise ssl_error("Underlying socket has been closed.")
         elif sockstate == SOCKET_TOO_LARGE_FOR_SELECT:
             raise ssl_error("Underlying socket too large for select().")
 
+        count = ffi.new("size_t[1]", [0])
         while True:
-            length = lib.SSL_write(self.ssl, b, lgt)
-            err = _PySSL_errno(length<=0, self.ssl, length)
+            retval = lib.SSL_write_ex(self.ssl, b, lgt, count)
+            err = _PySSL_errno(retval==0, self.ssl, retval)
             self.err = err
 
             check_signals()
@@ -589,7 +637,7 @@ class _SSLSocket(object):
                 sockstate = SOCKET_OPERATION_OK
 
             if sockstate == SOCKET_HAS_TIMED_OUT:
-                raise socket.timeout("The write operation timed out")
+                raise TimeoutError("The write operation timed out")
             elif sockstate == SOCKET_HAS_BEEN_CLOSED:
                 raise ssl_error("Underlying socket has been closed.")
             elif sockstate == SOCKET_IS_NONBLOCKING:
@@ -597,10 +645,9 @@ class _SSLSocket(object):
             if not (err.ssl == SSL_ERROR_WANT_READ or err.ssl == SSL_ERROR_WANT_WRITE):
                 break
 
-        if length > 0:
-            return length
-        else:
-            raise pyssl_error(self, length)
+        if retval ==0:
+            raise pyssl_error(self, retval)
+        return count[0]
 
     def read(self, length, buffer_into=None):
         if length < 0 and buffer_into is None:
@@ -616,6 +663,7 @@ class _SSLSocket(object):
 
         if length == 0:
             return b""
+        count = ffi.new("size_t[1]", [0])
         with StackNew("char[]", length) as dest:
             mem = dest
 
@@ -634,8 +682,8 @@ class _SSLSocket(object):
 
             shutdown = False
             while True:
-                count = lib.SSL_read(self.ssl, mem, length)
-                err = _PySSL_errno(count<=0, self.ssl, count)
+                retval = lib.SSL_read_ex(self.ssl, mem, length, count)
+                err = _PySSL_errno(retval==0, self.ssl, count[0])
                 self.err = err
 
                 check_signals()
@@ -649,22 +697,23 @@ class _SSLSocket(object):
                     sockstate = _ssl_select(sock, 1, timeout)
                 elif err.ssl == SSL_ERROR_ZERO_RETURN and \
                      lib.SSL_get_shutdown(self.ssl) == lib.SSL_RECEIVED_SHUTDOWN:
-                    shutdown = True
+                    count[0] = 0
+                    retval = 1  # in _ssl.c this is 'goto done' to avoid raising an error
                     break;
                 else:
                     sockstate = SOCKET_OPERATION_OK
 
                 if sockstate == SOCKET_HAS_TIMED_OUT:
-                    raise socket.timeout("The read operation timed out")
+                    raise TimeoutError("The read operation timed out")
                 elif sockstate == SOCKET_IS_NONBLOCKING:
                     break
                 if not (err.ssl == SSL_ERROR_WANT_READ or err.ssl == SSL_ERROR_WANT_WRITE):
                     break
 
-            if count <= 0 and not shutdown:
-                raise pyssl_error(self, count)
+            if retval == 0:
+                raise pyssl_error(self, retval)
 
-            return _bytes_with_len(dest, count)
+            return _bytes_with_len(dest, count[0])
 
     def _read_buf(self, length, buffer_into):
         ssl = self.ssl
@@ -691,9 +740,10 @@ class _SSLSocket(object):
             deadline = _monotonic_clock() + timeout
 
         shutdown = False
+        count = ffi.new("size_t[1]", [0])
         while True:
-            count = lib.SSL_read(self.ssl, mem, length);
-            err = _PySSL_errno(count<=0, self.ssl, count)
+            retval = lib.SSL_read_ex(self.ssl, mem, length, count);
+            err = _PySSL_errno(retval==0, self.ssl, count[0])
             self.err = err
 
             check_signals()
@@ -707,22 +757,22 @@ class _SSLSocket(object):
                 sockstate = _ssl_select(sock, 1, timeout)
             elif err.ssl == SSL_ERROR_ZERO_RETURN and \
                  lib.SSL_get_shutdown(self.ssl) == lib.SSL_RECEIVED_SHUTDOWN:
-                shutdown = True
+                count[0] = 0
                 break;
             else:
                 sockstate = SOCKET_OPERATION_OK
 
             if sockstate == SOCKET_HAS_TIMED_OUT:
-                raise socket.timeout("The read operation timed out")
+                raise TimeoutError("The read operation timed out")
             elif sockstate == SOCKET_IS_NONBLOCKING:
                 break
             if not (err.ssl == SSL_ERROR_WANT_READ or err.ssl == SSL_ERROR_WANT_WRITE):
                 break
 
-        if count <= 0 and not shutdown:
-            raise pyssl_error(self, count)
+        if retval == 0:
+            raise pyssl_error(self, retval)
 
-        return count
+        return count[0]
 
     if HAS_ALPN:
         def selected_alpn_protocol(self):
@@ -735,13 +785,28 @@ class _SSLSocket(object):
             return _str_with_len(out[0], outlen[0]);
 
     def shared_ciphers(self):
-        ciphers = lib.SSL_get_ciphers(self.ssl)
-        if ciphers == ffi.NULL:
+
+        # Rather than use SSL_get_shared_ciphers, we use an equivalent
+        # algorithm because:
+        # 1) It returns a colon seperated list of strings, in an undefined
+        #    order, that we would have to post process back into tuples.
+        # 2) It will return a truncated string with no indication that it has
+        #    done so, if the buffer is too small.
+
+
+        server_ciphers = lib.SSL_get_ciphers(self.ssl)
+        if server_ciphers == ffi.NULL:
+            return None
+        client_ciphers = lib.SSL_get_client_ciphers(self.ssl)
+        if client_ciphers == ffi.NULL:
             return None
         res = []
-        count = lib.sk_SSL_CIPHER_num(ciphers)
+        count = lib.sk_SSL_CIPHER_num(server_ciphers)
         for i in range(count):
-            tup = cipher_to_tuple(lib.sk_SSL_CIPHER_value(ciphers, i))
+            cipher = lib.sk_SSL_CIPHER_value(server_ciphers, i)
+            if lib.sk_SSL_CIPHER_find(client_ciphers, cipher) < 0:
+                continue
+            tup = cipher_to_tuple(cipher)
             if not tup:
                 return None
             res.append(tup)
@@ -807,9 +872,8 @@ class _SSLSocket(object):
 
             timeout = _socket_timeout(sock)
             nonblocking = timeout >= 0
-            if sock and timeout >= 0:
-                lib.BIO_set_nbio(lib.SSL_get_rbio(ssl), nonblocking)
-                lib.BIO_set_nbio(lib.SSL_get_wbio(ssl), nonblocking)
+            lib.BIO_set_nbio(lib.SSL_get_rbio(ssl), nonblocking)
+            lib.BIO_set_nbio(lib.SSL_get_wbio(ssl), nonblocking)
         else:
             timeout = 0
 
@@ -838,8 +902,8 @@ class _SSLSocket(object):
                 break
             if ret == 0:
                 # Don't loop endlessly; instead preserve legacy
-                #   behaviour of trying SSL_shutdown() only twice.
-                #   This looks necessary for OpenSSL < 0.9.8m
+                # behaviour of trying SSL_shutdown() only twice.
+                # This looks necessary for OpenSSL < 0.9.8m
                 zeros += 1
                 if zeros > 1:
                     break
@@ -860,9 +924,9 @@ class _SSLSocket(object):
 
             if sockstate == SOCKET_HAS_TIMED_OUT:
                 if err.ssl == SSL_ERROR_WANT_READ:
-                    raise socket.timeout("The read operation timed out")
+                    raise TimeoutError("The read operation timed out")
                 else:
-                    raise socket.timeout("The write operation timed out")
+                    raise TimeoutError("The write operation timed out")
             elif sockstate == SOCKET_TOO_LARGE_FOR_SELECT:
                 raise ssl_error("Underlying socket too large for select().")
             elif sockstate != SOCKET_OPERATION_OK:
@@ -943,7 +1007,7 @@ class _SSLSocket(object):
             raise ValueError("Session refers to a different SSLContext.")
         if self.socket_type != SSL_CLIENT:
             raise ValueError("Cannot set session for server-side SSLSocket.")
-        if self.handshake_done:
+        if lib.SSL_is_init_finished(self.ssl):
             raise ValueError("Cannot set session after handshake.")
         if not lib.SSL_set_session(self.ssl, value._session):
             raise pyssl_error(self, 0)
@@ -957,11 +1021,15 @@ class _SSLSocket(object):
 
 def _fs_decode(name):
     return name.decode(sys.getfilesystemencoding())
-def _fs_converter(name):
+def _fs_converter(name, argname):
     """ name must not be None """
+    if isinstance(name, pathlib.Path):
+        name = str(name)
     if isinstance(name, str):
         return name.encode(sys.getfilesystemencoding())
-    return bytes(name)
+    elif isinstance(name, bytes):     
+        return bytes(name)
+    raise TypeError(f"{argname} should be a valid filesystem path")
 
 
 def cipher_to_tuple(cipher):
@@ -1084,16 +1152,21 @@ class _SSLContext(object):
         self._sni_cb = None
         self._msg_cb = None
         if protocol == PROTOCOL_TLSv1:
+            warnings.warn("ssl.PROTOCOL_TLSv1 is deprecated", DeprecationWarning)
             method = lib.TLSv1_method()
         elif lib.Cryptography_HAS_TLSv1_1 and protocol == PROTOCOL_TLSv1_1:
+            warnings.warn("ssl.PROTOCOL_TLSv1_1 is deprecated", DeprecationWarning)
             method = lib.TLSv1_1_method()
         elif lib.Cryptography_HAS_TLSv1_2 and protocol == PROTOCOL_TLSv1_2 :
+            warnings.warn("ssl.PROTOCOL_TLSv1_2 is deprecated", DeprecationWarning)
             method = lib.TLSv1_2_method()
         elif SSLv3_method_ok and protocol == PROTOCOL_SSLv3:
+            warnings.warn("ssl.PROTOCOL_SSLv3 is deprecated", DeprecationWarning)
             method = lib.SSLv3_method()
         elif lib.Cryptography_HAS_SSL2 and protocol == PROTOCOL_SSLv2:
             method = lib.SSLv2_method()
         elif protocol == PROTOCOL_SSLv23:
+            warnings.warn("ssl.PROTOCOL_TLS is deprecated", DeprecationWarning)
             method = lib.TLS_method()
         elif protocol == PROTOCOL_TLS_CLIENT:
             method = lib.TLS_client_method()
@@ -1102,10 +1175,10 @@ class _SSLContext(object):
         else:
             raise ValueError("invalid protocol version")
 
-        ctx = lib.SSL_CTX_new(method)
-        if ctx == ffi.NULL:
-            raise ssl_error("failed to allocate SSL context")
         self.ctx = ffi.gc(lib.SSL_CTX_new(method), lib.SSL_CTX_free)
+        __pypy__.add_memory_pressure(1000)
+        if self.ctx == ffi.NULL:
+            raise ssl_error("failed to allocate SSL context")
         self._post_handshake_auth = 0;
         self._protocol = protocol
         self.hostflags = lib.X509_CHECK_FLAG_NO_PARTIAL_WILDCARDS
@@ -1129,22 +1202,15 @@ class _SSLContext(object):
         options |= lib.SSL_OP_CIPHER_SERVER_PREFERENCE
         options |= lib.SSL_OP_SINGLE_DH_USE
         options |= lib.SSL_OP_SINGLE_ECDH_USE
-        if lib.Crytpography_HAS_OP_IGNORE_UNEXPECTED_EOF:
-            options |= lib.SSL_OP_IGNORE_UNEXPECTED_EOF
         lib.SSL_CTX_set_options(self.ctx, options)
-        lib.SSL_CTX_set_session_id_context(self.ctx, b"Python", len(b"Python"))
 
         # A bare minimum cipher list without completely broken cipher suites.
         # It's far from perfect but gives users a better head start.
-        if lib.Cryptography_HAS_SSL2 and protocol == PROTOCOL_SSLv2:
-            # SSLv2 needs MD5
-            default_ciphers = b"HIGH:!aNULL:!eNULL"
-        else:
-            default_ciphers = b"HIGH:!aNULL:!eNULL:!MD5"
-        if not lib.SSL_CTX_set_cipher_list(ctx, default_ciphers):
+        if not lib.SSL_CTX_set_cipher_list(self.ctx, PY_SSL_DEFAULT_CIPHER_STRING):
             lib.ERR_clear_error()
             raise SSLError("No cipher can be selected.")
 
+        lib.SSL_CTX_set_mode(self.ctx, lib.SSL_MODE_RELEASE_BUFFERS)
         if HAS_ECDH:
             # Allow automatic ECDH curve selection (on
             # OpenSSL 1.0.2+), or use prime256v1 by default.
@@ -1156,6 +1222,7 @@ class _SSLContext(object):
                 key = lib.EC_KEY_new_by_curve_name(lib.NID_X9_62_prime256v1)
                 lib.SSL_CTX_set_tmp_ecdh(self.ctx, key)
                 lib.EC_KEY_free(key)
+        lib.SSL_CTX_set_session_id_context(self.ctx, b"Python", len(b"Python"))
         params = lib.SSL_CTX_get0_param(self.ctx);
         if lib.Cryptography_HAS_X509_V_FLAG_TRUSTED_FIRST:
             # Improve trust chain building when cross-signed intermediate
@@ -1163,6 +1230,7 @@ class _SSLContext(object):
             lib.X509_VERIFY_PARAM_set_flags(params, lib.X509_V_FLAG_TRUSTED_FIRST)
         lib.X509_VERIFY_PARAM_set_hostflags(params, self.hostflags);
         if HAS_TLSv1_3:
+            self._post_handshake_auth = 0
             lib.SSL_CTX_set_post_handshake_auth(self.ctx, self.post_handshake_auth)
         return self
 
@@ -1188,10 +1256,17 @@ class _SSLContext(object):
 
     @options.setter
     def options(self, value):
+        opt_no = (
+            lib.SSL_OP_NO_SSLv2 | lib.SSL_OP_NO_SSLv3 | lib.SSL_OP_NO_TLSv1 |
+            lib.SSL_OP_NO_TLSv1_1 | lib.SSL_OP_NO_TLSv1_2 | lib.SSL_OP_NO_TLSv1_3
+        )
         new_opts = int(value)
         opts = lib.SSL_CTX_get_options(self.ctx)
         clear = opts & ~new_opts
         set = ~opts & new_opts
+        if set & opt_no != 0:
+            warnings.warn("ssl.OP_NO_SSL*/ssl.OP_NO_TLS* options are "
+                          "deprecated", DeprecationWarning, stacklevel=2)
         if clear:
             if lib.Cryptography_HAS_SSL_CTX_CLEAR_OPTIONS:
                 lib.SSL_CTX_clear_options(self.ctx, clear)
@@ -1266,6 +1341,20 @@ class _SSLContext(object):
                                      PROTOCOL_TLS):
                 raise ValueError("The context's protocol doesn't support"
                     "modification of highest and lowest version.")
+
+            # check for deprecations and supported values
+            if v == PROTO_SSLv3:
+                warnings.warn("ssl.TLSVersion.SSLv3 is deprecated", DeprecationWarning)
+            elif v == PROTO_TLSv1:
+                warnings.warn("ssl.TLSVersion.TLSv1 is deprecated", DeprecationWarning)
+            elif v == PROTO_TLSv1_1:
+                warnings.warn("ssl.TLSVersion.TLSv1_1 is deprecated", DeprecationWarning)
+            elif v in (PROTO_MINIMUM_SUPPORTED, PROTO_MAXIMUM_SUPPORTED,
+                       PROTO_TLSv1_2, PROTO_TLSv1_3):
+                pass
+            else:
+                raise ValueError(f"Unsupported TLS/SSL version '{v}'")
+
             if what == 0:
                 if v == PROTO_MINIMUM_SUPPORTED:
                     v = 0
@@ -1282,7 +1371,7 @@ class _SSLContext(object):
                 result = lib.SSL_CTX_set_max_proto_version(self.ctx, v)
             if result == 0:
                 raise ValueError('Unsupported protocol version 0x%x' % v)
-            return 0
+            return result
 
         @property
         def minimum_version(self):
@@ -1385,7 +1474,7 @@ class _SSLContext(object):
         prev_errno = ffi.errno
         try:
             ffi.errno = 0
-            certfilebuf = _str_to_ffi_buffer(certfile)
+            certfilebuf = _str_to_ffi_buffer(_fs_converter(certfile, "certfile"))
             ret = lib.SSL_CTX_use_certificate_chain_file(self.ctx, certfilebuf)
             if ret != 1:
                 if pw_info.operationerror:
@@ -1399,7 +1488,7 @@ class _SSLContext(object):
                     raise ssl_error(None)
 
             ffi.errno = 0
-            buf = _str_to_ffi_buffer(keyfile)
+            buf = _str_to_ffi_buffer(_fs_converter(keyfile, "keyfile"))
             ret = lib.SSL_CTX_use_PrivateKey_file(self.ctx, buf,
                                                   lib.SSL_FILETYPE_PEM)
             if ret != 1:
@@ -1458,11 +1547,11 @@ class _SSLContext(object):
                 if cafile is None:
                     cafilebuf = ffi.NULL
                 else:
-                    cafilebuf = _str_to_ffi_buffer(cafile)
+                    cafilebuf = _str_to_ffi_buffer(_fs_converter(cafile, "cafile"))
                 if capath is None:
                     capathbuf = ffi.NULL
                 else:
-                    capathbuf = _str_to_ffi_buffer(capath)
+                    capathbuf = _str_to_ffi_buffer(_fs_converter(capath, "capath"))
                 ret = lib.SSL_CTX_load_verify_locations(self.ctx, cafilebuf, capathbuf)
                 if ret != 1:
                     _errno = ffi.errno
@@ -1586,7 +1675,9 @@ class _SSLContext(object):
         x509 = 0
         x509_ca = 0
         crl = 0
-        objs = lib.X509_STORE_get0_objects(store)
+        objs = lib.X509_STORE_get1_objects(store)
+        if not objs:
+            raise MemoryError("failed to query cert store")
         count = lib.sk_X509_OBJECT_num(objs)
         for i in range(count):
             obj = lib.sk_X509_OBJECT_value(objs, i)
@@ -1599,10 +1690,9 @@ class _SSLContext(object):
             elif _type == lib.X509_LU_CRL:
                 crl += 1
             else:
-                # Ignore X509_LU_FAIL, X509_LU_RETRY, X509_LU_PKEY.
-                # As far as I can tell they are internal states and never
-                # stored in a cert store
+                # Ignore unrecognized types
                 pass
+        lib.sk_X509_OBJECT_pop_free(objs, lib.X509_OBJECT_free);
         return {'x509': x509, 'x509_ca': x509_ca, 'crl': crl}
 
 
@@ -1629,7 +1719,7 @@ class _SSLContext(object):
             ffi.errno = 0
             if filepath is None:
                 raise TypeError("filepath must not be None")
-            buf = _fs_converter(filepath)
+            buf = _fs_converter(filepath, "filepath")
             mode = ffi.new("char[]",b"rb")
             ffi.errno = 0
             bio = lib.BIO_new_file(buf, mode)
@@ -1660,29 +1750,34 @@ class _SSLContext(object):
         binary_mode = bool(binary_form)
         _list = []
         store = lib.SSL_CTX_get_cert_store(self.ctx)
-        objs = lib.X509_STORE_get0_objects(store)
-        count = lib.sk_X509_OBJECT_num(objs)
-        for i in range(count):
-            obj = lib.sk_X509_OBJECT_value(objs, i)
-            _type = lib.X509_OBJECT_get_type(obj)
-            if _type != lib.X509_LU_X509:
-                # not a x509 cert
-                continue
-            # CA for any purpose
-            cert = lib.X509_OBJECT_get0_X509(obj)
-            if not lib.X509_check_ca(cert):
-                continue
-            if binary_mode:
-                _list.append(_certificate_to_der(cert))
-            else:
-                _list.append(_decode_certificate(cert))
+        objs = lib.X509_STORE_get1_objects(store)
+        if not objs:
+            raise MemoryError("failed to query cert store")
+        try:
+            count = lib.sk_X509_OBJECT_num(objs)
+            for i in range(count):
+                obj = lib.sk_X509_OBJECT_value(objs, i)
+                _type = lib.X509_OBJECT_get_type(obj)
+                if _type != lib.X509_LU_X509:
+                    # not a x509 cert
+                    continue
+                # CA for any purpose
+                cert = lib.X509_OBJECT_get0_X509(obj)
+                if not lib.X509_check_ca(cert):
+                    continue
+                if binary_mode:
+                    _list.append(_certificate_to_der(cert))
+                else:
+                    _list.append(_decode_certificate(cert))
+        finally:
+            lib.sk_X509_OBJECT_pop_free(objs, lib.X509_OBJECT_free);
         return _list
 
     def set_ecdh_curve(self, name):
         # needs to be zero terminated
         if name is None:
             raise TypeError()
-        buf = _fs_converter(name)
+        buf = _fs_converter(name, "name")
         nid = lib.OBJ_sn2nid(buf)
         if nid == 0:
             raise ValueError("unknown elliptic curve name '%s'" % name)
@@ -2007,6 +2102,7 @@ def _RAND_bytes(count, pseudo):
     raise ssl_error(None, errcode=lib.ERR_get_error())
 
 def RAND_pseudo_bytes(count):
+    warnings.warn("ssl.RAND_pseudo_bytes() is deprecated", DeprecationWarning)
     return _RAND_bytes(count, True)
 
 def RAND_bytes(count):
