@@ -1,7 +1,7 @@
 """
 Support routines for overlapping io.
 Currently, this extension module is only required when using the
-modules on Windows.
+modules on Windows. It is used in asyncio
 """
 
 import sys
@@ -177,6 +177,10 @@ def initiailize_function_ptrs():
 
 initiailize_function_ptrs()
 
+# This is the class used in asyncio. It is different than the one used
+# in multiprocessing, which is in _winapi. See
+# https://github.com/python/cpython/issues/64613 for a tiny discussion of
+# merging the two
 
 class Overlapped(object):
     def __init__(self, event=_ffi.NULL):
@@ -272,6 +276,19 @@ class Overlapped(object):
         self.read_buffer = _ffi.new("CHAR[]", max(1, size))
         return self.do_WSARecv(handle, self.read_buffer, size, flags)
 
+    def WSARecvInto(self, handle, bufobj, flags):
+        handle = _int2handle(handle)
+        flags = _int2dword(flags)
+        if self.type != OverlappedType.TYPE_NONE:
+            _winapi.raise_WinError()
+        if not isinstance(bufobj, bytearray):
+            raise TypeError("bytearray expected in WSARecvInto")
+        size = len(bufobj)
+        self.type = OverlappedType.TYPE_READINTO
+        self.handle = handle
+        self.read_buffer = _ffi.from_buffer(bufobj)
+        return self.do_WSARecv(handle, self.read_buffer, size, flags)
+
     def do_WSARecv(self, handle, allocatedbuffer, size, flags):
         nread = _ffi.new("LPDWORD")
         wsabuff = _ffi.new("WSABUF[1]")
@@ -359,6 +376,18 @@ class Overlapped(object):
         self.read_buffer = _ffi.new("CHAR[]", max(1, size))
         return self.do_ReadFile(self.handle, self.read_buffer, size)
 
+    def ReadFileInto(self, handle, bufobj):
+        handle = _int2handle(handle)
+        if self.type != OverlappedType.TYPE_NONE:
+            _winapi.raise_WinError()
+        if not isinstance(bufobj, bytearray):
+            raise TypeError("bytearray expected in ReadFileInto")
+        size = len(bufobj)
+        self.type = OverlappedType.TYPE_READINTO
+        self.handle = handle
+        self.read_buffer = _ffi.from_buffer(bufobj)
+        return self.do_ReadFile(handle, self.read_buffer, size)
+
     def do_ReadFile(self, handle, buf, size):
         nread = _ffi.new('DWORD[1]', [0])
         ret = _kernel32.ReadFile(handle, buf, size, nread, self.overlapped)
