@@ -23,14 +23,15 @@
 #
 # Note: This software has been modified for use in PyPy.
 
-from collections import OrderedDict
-from functools import wraps
 import datetime
+import os
 import string
 import sys
-import weakref
 import threading
-import os
+import types
+import weakref
+from collections import OrderedDict
+from functools import wraps
 
 try:
     from __pypy__ import newlist_hint, add_memory_pressure
@@ -104,14 +105,11 @@ PARSE_DECLTYPES = 2
 # SQLite version information
 sqlite_version = str(_ffi.string(_lib.sqlite3_libversion()).decode('ascii'))
 
-_STMT_TYPE_UPDATE = 0
-_STMT_TYPE_DELETE = 1
-_STMT_TYPE_INSERT = 2
-_STMT_TYPE_REPLACE = 3
-_STMT_TYPE_OTHER = 4
-_STMT_TYPE_SELECT = 5
-_STMT_TYPE_INVALID = 6
+# flag that signals if base types need adaption
+BASE_TYPE_ADAPTED = False
 
+# set of base types that are supported by SQLite3
+BASE_TYPES = {bytearray, bytes, float, int, str, NoneType}
 
 class Error(StandardError):
     pass
@@ -168,16 +166,6 @@ def connect(database, timeout=5.0, detect_types=0, isolation_level="",
 def _unicode_text_factory(x):
     return unicode(x, 'utf-8')
 
-if sys.version_info[0] < 3:
-    def OptimizedUnicode(s):
-        try:
-            val = unicode(s, "ascii").encode("ascii")
-        except UnicodeDecodeError:
-            val = unicode(s, "utf-8")
-        return val
-else:
-    OptimizedUnicode = _unicode_text_factory
-
 
 class _StatementCache(object):
     def __init__(self, connection, maxcount):
@@ -186,20 +174,14 @@ class _StatementCache(object):
         self.cache = OrderedDict()
 
     def get(self, sql):
-        notincache = object()
-        try:
-            stat = self.cache[sql]
-        except KeyError:
-            stat = notincache
-        if stat is notincache:
-            stat = Statement(self.connection, sql)
+        stat = self.cache.get(sql, None)
+        not_there = stat is None
+        if not_there or stat._in_use_token:
+            stat = object.__new__(Statement)
+            stat.__init__(self.connection, sql)
             self.cache[sql] = stat
-            if len(self.cache) > self.maxcount:
+            if not_there and len(self.cache) > self.maxcount:
                 self.cache.popitem(0)
-        else:
-            if stat._in_use_token:
-                stat = Statement(self.connection, sql)
-                self.cache[sql] = stat
         return stat
 
 BEGIN_STATMENTS = (
@@ -792,8 +774,11 @@ class Connection(object):
 class Cursor(object):
     __initialized = False
     __statement = None
+    __locked = False
 
     def __init__(self, con):
+        if self.__locked:
+            raise ProgrammingError("Recursive use of cursors not allowed.")
         if not isinstance(con, Connection):
             raise TypeError
         self.__connection = con
@@ -823,6 +808,8 @@ class Cursor(object):
     def close(self):
         if not self.__initialized:
             raise ProgrammingError("Base Cursor.__init__ not called.")
+        if self.__locked:
+            raise ProgrammingError("Recursive use of cursors not allowed.")
         self.__connection._check_thread()
         self.__connection._check_closed()
         if self.__statement:
@@ -1072,7 +1059,12 @@ class Cursor(object):
 
         ret = _lib.sqlite3_step(self.__statement._statement)
         if ret == _lib.SQLITE_ROW:
-            self.__next_row = self.__fetch_one_row()
+            assert not self.__locked
+            self.__locked = True
+            try:
+                self.__next_row = self.__fetch_one_row()
+            finally:
+                self.__locked = False
         else:
             self.__statement._reset(self.__in_use_token)
             if ret != _lib.SQLITE_DONE:
@@ -1112,9 +1104,26 @@ class Cursor(object):
         try:
             return self.__description
         except AttributeError:
-            if self.__statement:
-                self.__description = self.__statement._get_description()
-                return self.__description
+            if not self.__statement:
+                return None
+            statement = self.__statement
+            if not statement._valid:
+                return None
+
+            numcols = _lib.sqlite3_column_count(statement._statement)
+            if numcols <= 0:
+                self.__description = None
+                return None
+            desc = []
+            for i in xrange(numcols):
+                name = _lib.sqlite3_column_name(statement._statement, i)
+                if name:
+                    name = _ffi.string(name).decode('utf-8')
+                    if self.__connection._detect_types & PARSE_COLNAMES:
+                        name = name.split("[", 1)[0].strip()
+                desc.append((name, None, None, None, None, None, None))
+            self.__description = tuple(desc)
+            return self.__description
     description = property(__get_description)
 
     def __get_lastrowid(self):
@@ -1171,6 +1180,9 @@ class Statement(object):
         if _check_remaining_sql(tail):
             raise Warning("You can only execute one statement at a time.")
 
+    def __new__(cls, *args):
+        raise TypeError("cannot create '_sqlite3.Statement' instances")
+
     def __del__(self):
         if self._statement:
             self.__con._finalize_raw_statement(self._statement)
@@ -1193,11 +1205,16 @@ class Statement(object):
             self._in_use_token = None
 
     def __set_param(self, idx, param):
-        cvt = converters.get(type(param))
-        if cvt is not None:
-            param = cvt(param)
-
-        param = adapt(param)
+        typ = type(param)
+        if BASE_TYPE_ADAPTED or (
+            typ is not bytearray
+            and typ is not bytes
+            and typ is not float
+            and typ is not int
+            and typ is not str
+            and typ is not NoneType
+        ):
+            param = adapt(param)
 
         if param is None:
             rc = _lib.sqlite3_bind_null(self._statement, idx)
@@ -1272,19 +1289,6 @@ class Statement(object):
         else:
             raise ValueError("parameters are of unsupported type")
 
-    def _get_description(self):
-        if self._is_dml or not self._valid:
-            return None
-        desc = []
-        for i in xrange(_lib.sqlite3_column_count(self._statement)):
-            name = _lib.sqlite3_column_name(self._statement, i)
-            if name:
-                name = _ffi.string(name).decode('utf-8')
-                if self.__con._detect_types & PARSE_COLNAMES:
-                    name = name.split("[")[0].strip()
-            desc.append((name, None, None, None, None, None, None))
-        return desc
-
 
 class Row(object):
     def __init__(self, cursor, values):
@@ -1439,6 +1443,11 @@ class PrepareProtocol(object):
 
 
 def register_adapter(typ, callable):
+    global BASE_TYPE_ADAPTED
+
+    if typ in BASE_TYPES:
+        BASE_TYPE_ADAPTED = True
+
     adapters[typ, PrepareProtocol] = callable
 
 
@@ -1502,4 +1511,7 @@ def adapt(val, proto=PrepareProtocol):
 
     return val
 
+def enable_shared_cache(enable):
+    raise OperationalError("enable_shared_cache not supported on PyPy (and it's deprecated anyway)")
+
 register_adapters_and_converters()
